Package: future.tests
Title: Test Suite for 'Future API' Backends
Version: 1.0.0
Authors@R: c(
    person("Henrik", "Bengtsson", role = c("aut", "cre", "cph"), email = "henrikb@braju.com",
           comment = c(ORCID = "0000-0002-7579-5165")),
    person(family = "R Consortium", role = "fnd", comment = c(ROR = "01z833950"))
  )
Description: Backends implementing the 'Future' API <doi:10.32614/RJ-2021-048>, as defined by the 'future' package, should use the tests provided by this package to validate that they meet the minimal requirements of the 'Future' API.  The tests can be performed easily from within R or from outside of R from the command line making it straightforward to include them in package tests and in Continuous Integration (CI) pipelines.
Imports: future (>= 1.40.0), cli, crayon, prettyunits, sessioninfo
Suggests: commonmark, base64enc, utils, tools, data.table, ff
VignetteBuilder: future.tests
License: Apache License (>= 2)
Encoding: UTF-8
URL: https://future.tests.futureverse.org,
        https://github.com/futureverse/future.tests
BugReports: https://github.com/futureverse/future.tests/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-15 11:21:21 UTC; hb
Author: Henrik Bengtsson [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-7579-5165>),
  R Consortium [fnd] (ROR: <https://ror.org/01z833950>)
Maintainer: Henrik Bengtsson <henrikb@braju.com>
Repository: CRAN
Date/Publication: 2026-07-15 12:10:02 UTC
Built: R 4.6.1; ; 2026-07-15 13:28:36 UTC; unix
