# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,Test)
S3method(as.data.frame,TestResult)
S3method(print,Test)
S3method(print,TestResult)
S3method(print,cli_fcn)
S3method(rbind,Test)
S3method(rbind,TestResult)
export(add_test_plan)
export(along_test_plans)
export(check)
export(check_plan)
export(evaluate_expr)
export(load_tests)
export(make_test)
export(register_test)
export(run_test)
export(run_tests)
export(skip_test)
export(subset_tests)
export(test_db)
export(test_plans)
importFrom(cli,get_spinner)
importFrom(cli,rule)
importFrom(cli,symbol)
importFrom(crayon,blue)
importFrom(crayon,cyan)
importFrom(crayon,green)
importFrom(crayon,red)
importFrom(crayon,silver)
importFrom(crayon,yellow)
importFrom(future,availableCores)
importFrom(future,plan)
importFrom(future,resetWorkers)
importFrom(future,sequential)
importFrom(grDevices,dev.list)
importFrom(grDevices,dev.off)
importFrom(prettyunits,pretty_dt)
importFrom(prettyunits,pretty_sec)
importFrom(sessioninfo,session_info)
importFrom(utils,capture.output)
importFrom(utils,file_test)
importFrom(utils,packageVersion)
importFrom(utils,str)
